﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;

namespace SSO.Server.Code
{
    /************************************************************************
    * 版 本 号 :  v1.0.0.0
    * 作    者 :  yinchenghui
    * 创建时间 :  2019/3/25 16:12:37
    * 更新时间 :  2019/3/25 16:12:37
    * 描    述 :  登录逻辑
    ************************************************************************/
    public class LoginUtils
    {
        const int expireMin = 20;//登陆过期时间（分钟）
        const string LOGINNAME = "SSO-LoginName";//登录用户名
        const string LOGINTOKEN = "SSO-LoginToken";//登陆Token
        /// <summary>
        /// 登陆
        /// </summary>
        /// <param name="userName"></param>
        /// <param name="loginInfo"></param>
        public static void Login(string userName, LoginInfo loginInfo)
        {
            string loginToken = Guid.NewGuid().ToString("n");

            WriteCookie(LOGINNAME, userName);
            WriteCookie(LOGINTOKEN, loginToken);

            loginInfo.UserName = userName;
            loginInfo.LoginToken = loginToken;
            loginInfo.SsoTicket = Guid.NewGuid().ToString("n");
            WriteCache(GetLoginCacheKey(userName), loginInfo, new TimeSpan(0, expireMin, 0));
        }
        /// <summary>
        /// 登出
        /// </summary>
        public static void Logout()
        {
            //移除缓存
            string userName = GetCookie(LOGINNAME);
            RemoveCache(GetLoginCacheKey(userName));
        }
        /// <summary>
        /// 登出
        /// </summary>
        /// <param name="userName"></param>
        public static void Logout(string userName)
        {
            RemoveCache(GetLoginCacheKey(userName));
        }
        /// <summary>
        /// 是否登录状态
        /// </summary>
        /// <returns></returns>
        public static bool IsLogin()
        {
            string userName = GetCookie(LOGINNAME);
            string loginToken = GetCookie(LOGINTOKEN);
            if (!string.IsNullOrEmpty(userName) && !string.IsNullOrEmpty(loginToken))
            {
                object loginCache = GetCache(GetLoginCacheKey(userName));
                if (loginCache != null)
                {
                    var loginInfo = loginCache as LoginInfo;
                    if (loginInfo.LoginToken == loginToken)
                        return true;
                }
            }

            //发现是登录状态,则把登录cookie清掉
            RemoveCookie(LOGINNAME);
            RemoveCookie(LOGINTOKEN);
            return false;
        }
        /// <summary>
        /// 查登陆信息
        /// </summary>
        /// <returns></returns>
        public static LoginInfo GetLoginInfo()
        {
            string userName = GetCookie(LOGINNAME);
            object loginCache = GetCache(GetLoginCacheKey(userName));
            return loginCache as LoginInfo;
        }
        /// <summary>
        /// 查登陆信息
        /// </summary>
        /// <param name="userName"></param>
        /// <returns></returns>
        public static LoginInfo GetLoginInfo(string userName)
        {
            object loginCache = GetCache(GetLoginCacheKey(userName));
            if (loginCache != null)
            {
                return loginCache as LoginInfo;
            }
            return null;
        }
        /// <summary>
        /// 注册客户端app信息
        /// </summary>
        public static void RegistClientInfo(string userName, ClientInfo clientInfo)
        {
            LoginInfo loginInfo = GetLoginInfo();
            if (loginInfo.Clients == null) loginInfo.Clients = new HashSet<ClientInfo>();
            loginInfo.Clients.Add(clientInfo);
        }
        #region 私有方法
        /// <summary>
        /// 获取登陆缓存
        /// </summary>
        /// <returns></returns>
        private static string GetLoginCacheKey(string userName)
        {
            return string.Format("{0}:{1}", LOGINNAME, userName);
        }
        /// <summary>
        /// 写cookie值
        /// </summary>
        /// <param name="cookieName">名称</param>
        /// <param name="cookieValue">值</param>
        private static void WriteCookie(string cookieName, string cookieValue)
        {
            HttpCookie cookie = HttpContext.Current.Request.Cookies[cookieName];
            if (cookie == null)
            {
                cookie = new HttpCookie(cookieName);
            }
            cookie.Value = cookieValue;
            HttpContext.Current.Response.AppendCookie(cookie);
        }
        /// <summary>
        /// 读cookie值
        /// </summary>
        /// <param name="cookieName">名称</param>
        /// <returns>cookie值</returns>
        private static string GetCookie(string cookieName)
        {
            if (HttpContext.Current.Request.Cookies != null && HttpContext.Current.Request.Cookies[cookieName] != null)
            {
                return HttpContext.Current.Request.Cookies[cookieName].Value.ToString();
            }
            return "";
        }
        /// <summary>
        /// 删除Cookie对象
        /// </summary>
        /// <param name="CookiesName">Cookie对象名称</param>
        private static void RemoveCookie(string cookieName)
        {
            HttpCookie objCookie = new HttpCookie(cookieName.Trim());
            objCookie.Expires = DateTime.Now.AddYears(-5);
            HttpContext.Current.Response.Cookies.Add(objCookie);
        }
        /// <summary>
        /// 写缓存
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <param name="objObject"></param>
        /// <param name="expire"></param>
        private static void WriteCache(string cacheKey, object objObject, TimeSpan expire)
        {
            System.Web.Caching.Cache objCache = HttpContext.Current.Cache;
            objCache.Insert(cacheKey, objObject, null, DateTime.Now.Add(expire), Cache.NoSlidingExpiration);
        }
        /// <summary>
        /// 读缓存
        /// </summary>
        /// <param name="cacheKey"></param>
        /// <returns></returns>
        private static object GetCache(string cacheKey)
        {
            System.Web.Caching.Cache objCache = HttpContext.Current.Cache;
            return objCache[cacheKey];
        }
        /// <summary>
        /// 清除缓存
        /// </summary>
        /// <param name="cacheKey"></param>
        private static void RemoveCache(string cacheKey)
        {
            System.Web.Caching.Cache objCache = HttpContext.Current.Cache;
            objCache.Remove(cacheKey);
        }
        #endregion
    }
    public class LoginInfo
    {
        public string UserName { get; set; }
        /// <summary>
        /// sso 登陆token
        /// </summary>
        public string LoginToken { get; set; }
        /// <summary>
        /// 单点登录票据
        /// </summary>
        public string SsoTicket { get; set; }
        public HashSet<ClientInfo> Clients { get; set; }
    }
    /// <summary>
    /// 客户端信息
    /// </summary>
    public class ClientInfo
    {
        public string LogoutUrl { get; set; }
    }
}